#!/usr/bin/env python

from PySide import QtCore, QtGui
from exceptionwarning import ExceptionWarning as EW
import ui_generatekeydialog
import ui_generating
import gnupg
import os
import sptempdir
import gpgdir

def createTmpDir():
    gpgTempDir = sptempdir.getTempDir()
    print "Created directory %s" % gpgTempDir
    return gpgTempDir

def removeTmpDir(gpgTempDir):
    files = os.listdir(gpgTempDir)
    for f in files:
        fullName = os.path.join(gpgTempDir, f)
        os.unlink(fullName)
        print "Removed %s" % fullName

    os.rmdir(gpgTempDir)
    print "Removed %s" % gpgTempDir

class GenerateThread(QtCore.QThread):
    keysGenerated = QtCore.Signal(str, str)

    def __init__(self, keySize, name, email, passPhrase, dirName, parent):
        super(GenerateThread, self).__init__(parent)
        self.keySize = keySize
        self.passPhrase = passPhrase
        self.name = name
        self.email = email
        self.dirName = dirName

    def run(self):

        gpgTempDir = self.dirName
        name = self.name
        email = self.email
        passPhrase = self.passPhrase
        keyLength = self.keySize

        print "gpgTempDir", gpgTempDir

        gpg = gnupg.GPG(gpgbinary=gpgdir.gpg, gnupghome=gpgTempDir, verbose=True)
        input_data = gpg.gen_key_input(key_type="RSA", key_length=keyLength, name_real=name, name_email=email, passphrase=passPhrase)
        gpg.gen_key(input_data)

        privKeys = gpg.list_keys(True)
        ownFingerPrint = privKeys[0]["fingerprint"]

        pubKey = str(gpg.export_keys([ownFingerPrint]))
        privKey = str(gpg.export_keys([ownFingerPrint], True))

        self.keysGenerated.emit(pubKey, privKey)

class GeneratingDialog(QtGui.QDialog):
    def __init__(self, parent):
        super(GeneratingDialog, self).__init__(parent)

        self.ui = ui_generating.Ui_GeneratingDialog()
        self.ui.setupUi(self)

    def closeEvent(self, e):
        e.ignore()

    # Make sure escape doesn't close the dialog
    def keyPressEvent(self, e):
        e.accept()

class GenerateKeyDialog(QtGui.QDialog):
    def __init__(self, parent):
        super(GenerateKeyDialog, self).__init__(parent)

        self.ui = ui_generatekeydialog.Ui_GenerateKeyDialog()
        self.ui.setupUi(self)

        self.ui.generateButton.clicked.connect(EW(self.onGenerate))
        self.ui.cancelButton.clicked.connect(self.reject)

        self.pubKey = None
        self.privKey = None

    def onGenerate(self):
        passPhrase = self.ui.passphraseEdit.text()
        if passPhrase != self.ui.passPhraseRepeatEdit.text():
            QtGui.QMessageBox.warning(self, "Passphrases don't match", "The specified passphrases are not identical")
            return

        if not passPhrase:
            QtGui.QMessageBox.warning(self, "No passphrase", "You must specify a passphrase")
            return

        name = self.ui.nameEdit.text()
        email = self.ui.emailEdit.text()
        if not name or not email:
            QtGui.QMessageBox.warning(self, "No name or e-mail address", "You must specify both your name and an e-mail address")
            return

        keySize = int(self.ui.keysizeCombo.currentText())

        dlg = GeneratingDialog(self)
        gpgTempDir = createTmpDir()

        # Start generation in background thread
        
        genThread = GenerateThread(keySize, name, email, passPhrase, gpgTempDir, self)
        genThread.finished.connect(dlg.accept)
        genThread.keysGenerated.connect(EW(self.onKeysGenerated, False))
        genThread.start()

        # This should never be interrupted, killing the underlying thread/process seems to
        # make the application crash
        dlg.exec_()

        removeTmpDir(gpgTempDir)

        # If we get here, the keys should be generated successfully

        QtGui.QMessageBox.information(self, "Keys generated", "Keypair generated successfully")

        self.accept()

    def onKeysGenerated(self, pubKey, privKey):
        self.pubKey = pubKey
        self.privKey = privKey

    def getKeys(self):
        return (self.privKey, self.pubKey)

