#!/usr/bin/env python

from PySide import QtGui
from exceptionwarning import ExceptionWarning as EW
import ui_keydialog

class KeyDialog(QtGui.QDialog):
    def __init__(self, key, parent):
        super(KeyDialog, self).__init__(parent)

        self.ui = ui_keydialog.Ui_KeyDialog()
        self.ui.setupUi(self)

        self.ui.cancelButton.clicked.connect(self.reject)
        self.ui.importButton.clicked.connect(EW(self.onImport))

        if key: # Just view
            self.ui.keyEdit.setPlainText(key)
            self.ui.keyEdit.setReadOnly(True)
            self.ui.importButton.hide()
            self.ui.cancelButton.setText("Close")
            self.ui.infoLabel.setText("ASCII key info")

        self.key = key

    def onImport(self):
        keyStr = self.ui.keyEdit.toPlainText()
        if not "-----BEGIN PGP PRIVATE KEY BLOCK-----" in keyStr:
            QtGui.QMessageBox.warning(self, "Invalid key", "Specified key does not apear to be an (ASCII encoded) private key")
            return

        self.key = keyStr

        self.accept()

    def getKey(self):
        return self.key

