from PySide import QtGui, QtCore

class SherPassTableWidget(QtGui.QTableWidget):
    rightClickSignal = QtCore.Signal(int, int, int, int)

    def __init__(self, parent):
        super(SherPassTableWidget, self).__init__(parent)

    def mouseReleaseEvent(self, evt):

        super(SherPassTableWidget, self).mouseReleaseEvent(evt)
        
        if evt.button() == QtCore.Qt.RightButton:
            x = self.columnAt(evt.x())
            y = self.rowAt(evt.y())

            if x >= 0 and y >= 0:
                self.rightClickSignal.emit(y, x, evt.globalX(), evt.globalY())

    def hasFilter(self, filt, obj):
        if not filt:
            return False
        filt = filt.lower()

        if obj is None:
            return False

        if type(obj) == dict:
            for value in obj.values():
                if self.hasFilter(filt, value):
                    return True
        
        elif type(obj) == list:
            for value in obj:
                if self.hasFilter(filt, value):
                    return True

        else:
            if filt in str(obj):
                return True

        return False

    def setFilter(self, filt):
        rows = self.rowCount()
        for r in range(rows):
            item = self.item(r, 0)

            if filt:
                gotMatch = False
                entryData = item.entryData[1]
                for (field,value) in entryData.iteritems():
                    if field in [ "login", "host", "description", "extra" ]:
                        if self.hasFilter(filt, value):
                            gotMatch = True
                            break

                self.setRowHidden(r, not gotMatch)
            else:
                self.setRowHidden(r, False)
                        
    def keyPressEvent(self, evt):
        self.parentWidget().keyPressEvent(evt)

    def keyReleaseEvent(self, evt):
        self.parentWidget().keyReleaseEvent(evt)

