#!/usr/bin/env python

import urllib
import urllib2    
import cookielib            
import json
import pprint
import copy
import sys
import HTMLParser

def getFileName(url):
    idx = url.rfind("/")
    if idx >= 0:
        return url[idx+1:]
    return ""

class Parser(HTMLParser.HTMLParser):

    def init(self):
        self.inListItem = False
        self.inFileSize = False
        self.href = None
        self.fileSize = None
        self.entries = [ ]

    def getAttr(self, name, attrs):

        for (n, val) in attrs:
            if n == name:
                return val

        return None

    def handle_starttag(self, tag, attrs):
        if tag == "li":
            classValue = self.getAttr('class', attrs)
            if classValue and "browse-file" in classValue:
                self.inListItem = True
                self.inFileSize = False
                self.href = None
                self.fileSize = None
            
            return

        if not self.inListItem:
            return

        if tag == "span":
            classValue = self.getAttr('class', attrs)
            if classValue == "size":
                self.inFileSize = True

            return

        if tag == "a":
            classValue = self.getAttr('class', attrs)
            if classValue == "filename-link":
                self.href = self.getAttr('href', attrs)

    def handle_data(self, data):
        if not self.inListItem:
            return
        
        if not self.inFileSize:
            return

        self.fileSize = data

    def handle_endtag(self, tag):
        if tag == "li":
            if self.href: 
                self.entries.append( (self.href, self.fileSize) )

            self.inListItem = True
            self.inFileSize = False
            self.href = None
            self.fileSize = None
            return

        if tag == "span":
            self.inFileSize = False
            return

    def getEntries(self):
        return self.entries


class DBConnection(object):
    def __init__(self):
        self.cookieJar = cookielib.LWPCookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookieJar))
        self.headers =  {'User-agent' : 'Mozilla/5.0 (X11; Linux x86_64; rv:15.0) Gecko/20100101 Firefox/15.0.1' }
        self.urlBase = "https://www.dropbox.com/"

    def getDirContents(self, dirName):
        req = urllib2.Request(self.urlBase + dirName, None, self.headers)
        handle = self.opener.open(req)
        result = handle.read()
        handle.close()

        #print >>sys.stderr, result

        #return self.processResult0(result)
        return self.processResult1(result)

    def processResult1(self, result):
        parser = Parser()
        parser.init()
        parser.feed(result)
        parser.close()

        entries = parser.getEntries()
        dlIsNull = "?dl=0"

        fileNames = [ ]
        for (name,fileSize) in entries:
            if fileSize != "--": # Don't return info about directories

                if name.endswith(dlIsNull):
                    name = name[:-len(dlIsNull)]

                fileNames.append( (getFileName(name).lower(), name) )

        fileNames.sort()
        fileNames = [ n[1] for n in fileNames ]

        return fileNames

    def processResult0(self, result):

        fileURLs = set()

        startStr = '<div class="filename"><a href="'

        idx = result.find(startStr)
        while idx >= 0:
            idx += len(startStr)
            idx2 = result.find('"', idx)
            if idx2 >= 0:
                url = result[idx:idx2]
                if not "?" in url:
                    url = url.replace("https://www.dropbox.com/", "https://dl.dropboxusercontent.com/")

                    fileURLs.add(url)
                idx = result.find(startStr, idx2+1)
            else:
                idx = -1
        
        fileURLs = [ (getFileName(i),i) for i in fileURLs ]
        fileURLs.sort()
        fileURLs = [ i[1] for i in fileURLs ]

        return fileURLs

def getDirContents(dirName):
    dbConn = DBConnection()
    return dbConn.getDirContents(dirName)

def main():
    for i in sys.argv[1:]:
        print i
        print getDirContents(i)
        print
        print

if __name__ == "__main__":
    main()
